import os

default_root_path = os.path.expanduser("~" + os.sep + "Documents")
recursive = True
case_sensitive = False
match_phrase = False
extensions = ["py", "txt"]
print_depth = 1
debug = False


def readlines(path):
    """Return contents of text file as a list of lines."""
    with open(path) as file:
        try:
            return file.readlines()
        except UnicodeDecodeError:
            print(f" unicode error in {path}")
            return []


def index_and_search(dirpath, root_path_length, search_strings, depth, results, extensions):
    if debug:
        print(dirpath)
    try:
        filenames = os.listdir(dirpath)
    except PermissionError:
        return
    for filename in filenames:
        path = dirpath + os.sep + filename
        if os.path.isdir(path):
            if recursive:
                if depth < print_depth:
                    print(path)
                index_and_search(path, root_path_length, search_strings, depth + 1, results, extensions)
        else:
            if "." not in filename:
                continue
            extension = filename.split(".")[-1]
            if extension not in extensions:
                continue
            lines = readlines(path)
            results_list = []
            results_dict = {}
            for string in search_strings:
                results_dict[string] = 0
            for line in lines:
                if not case_sensitive:
                    line = line.lower()
                all_found = True
                for string in search_strings:
                    count = line.count(string)
                    if count > 0:
                        results_dict[string] += count
                    else:
                        all_found = False
                if all_found:
                    results_list.append(line)
            search_occurrences = 0
            search_weighting = 1
            for string in search_strings:
                count = results_dict[string]
                search_weighting *= count
                search_occurrences += count
            if search_weighting > 0:
                search_matches = len(results_list)
                search_result = f" {search_matches:4} {search_weighting:6} {search_occurrences:4} {path[root_path_length:]}"
                results.append(search_result)
                if debug:
                    print(f"{search_occurrences:4} {path[root_path_length:]}")
    if depth == 0:
        results.sort(reverse=True)
        print()
        print("FINAL RESULTS")
        for search_result in results:
            print(search_result)
        print(f" {len(results)} files found")


while True:
    print()
    search_string = input("Search string? ")
    if search_string == "":
        break
    if not case_sensitive:
        search_string = search_string.lower()
    if not match_phrase:
        search_strings = search_string.split()
    else:
        search_strings = [search_string]
    root_path = default_root_path
    print(f"using extensions {extensions}")
    results = []
    if os.path.isdir(root_path):
        index_and_search(root_path, len(root_path) + 1, search_strings, 0, results, extensions)
